<?php

namespace FiloBlu\CustomerBlocker\Helper;

class DataHelper
{
    const EMAIL_BLOCKER_ENABLE_CONFIG_PATH = 'filoblu_customer_blocker/block_by_email_address/enable';
    const EMAIL_BLOCKER_ADDRESSES_CONFIG_PATH = 'filoblu_customer_blocker/block_by_email_address/email_addresses_to_block';
    const CUSTOMER_BLOCKER_ADDRESSES_CONFIG_PATH = 'filoblu_customer_blocker/block_by_email_address/customer_addresses_to_block';

    protected $scopeConfig;

    private $shippingAddressFields = array();
    
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        $this->scopeConfig = $scopeConfig;

        $this->shippingAddressFields[] = 'vat_id';
        $this->shippingAddressFields[] = 'vat_is_valid';
        $this->shippingAddressFields[] = 'vat_request_id';
        $this->shippingAddressFields[] = 'vat_request_date';
        $this->shippingAddressFields[] = 'vat_request_success';
        $this->shippingAddressFields[] = 'customer_address_id';
        $this->shippingAddressFields[] = 'prefix';
        $this->shippingAddressFields[] = 'firstname';
        $this->shippingAddressFields[] = 'middlename';
        $this->shippingAddressFields[] = 'lastname';
        $this->shippingAddressFields[] = 'suffix';
        $this->shippingAddressFields[] = 'company';
        $this->shippingAddressFields[] = 'street';
        $this->shippingAddressFields[] = 'city';
        $this->shippingAddressFields[] = 'region';
        $this->shippingAddressFields[] = 'region_id';
        $this->shippingAddressFields[] = 'postcode';
        $this->shippingAddressFields[] = 'country_id';
        $this->shippingAddressFields[] = 'telephone';
        $this->shippingAddressFields[] = 'fax';
        $this->shippingAddressFields[] = 'email';
        $this->shippingAddressFields[] = 'address_type';
        $this->shippingAddressFields[] = 'parent_id';
    }

    protected function loadFilter() {
        $email_list = preg_split('/,|' . "\n" . '/', $this->scopeConfig->getValue(self::EMAIL_BLOCKER_ADDRESSES_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE));
        $validated_list = array();
        foreach ($email_list as $address) {
            $address = trim($address);
            if ($address != '') {
                $validated_list[] = $address;
            }
        }
        return $validated_list;
    }

    protected function loadFilterCustomerAddresses() {
        $address_list = preg_split('/,|' . "\n" . '/', $this->scopeConfig->getValue(self::CUSTOMER_BLOCKER_ADDRESSES_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE));
        $validated_list = array();
        foreach ($address_list as $address) {
            $address = trim($address);
            if ($address != '') {
                $validated_list[] = $address;
            }
        }
        return $validated_list;
    }

    public function mappingAddressData($data)
    {
        $address = array();

        foreach ($data as $key => $val)
        {
            if(in_array($key, $this->shippingAddressFields))
            {
                $address[$key] = $val;
            }
        }

        return $address;
    }

    public function isModuleEnable()
    {
        return $this->scopeConfig->isSetFlag(self::EMAIL_BLOCKER_ENABLE_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE);
    }

    public function isEmailToBeBlocked($email)
    {
        $filtered_email_addresses = $this->loadFilter();
        $emailToBlock = false;

        if (!empty($filtered_email_addresses))
        {
            $email = strtolower($email);
            foreach ($filtered_email_addresses as $filter) {
                $filter = str_replace('.', '\.', $filter);
                $filter = str_replace('+', '\+', $filter);
                $filter = str_replace('*', '(.*?)', $filter);
                $filter = strtolower($filter);
                $expr = '/^' . trim($filter) . '$/';
                //$expr = '/^' . trim(str_replace('*', '(.*?)', str_replace('.', '\.', $filter))) . '$/';
                if (preg_match($expr, $email)) {
                    //$this->messageManager->addError(__("Sorry, your order cannot be processed."));
                    $emailToBlock = true;
                    break;
                }
            }
        }

        return $emailToBlock;
    }

    public function isAddressToBeBlocked($data)
    {
        $addressToBlock = false;
        // Get address list to block
        $customer_address = $this->loadFilterCustomerAddresses();

        if(!empty($customer_address))
        {
            $orderData = [];
            foreach ($data as $key => $value)
            {
                $orderData[$key] = strtolower($this->sanitizeLetters($value));
            }

            // Cleaning of the 'Street' field to the order with special characters
            $orderStreet = preg_replace('/[^0-9a-zA-Z\s]/', '', $orderData['street']);
            $orderCity = preg_replace('/[^0-9a-zA-Z\s]/', '', $orderData['city']);
            $arrayOrderStreet = explode(" ", $orderStreet);
            $arrayOrderCity = explode(" ", $orderCity);
            foreach ($customer_address as $address)
            {
                $check = false;
                $arrayAddress = array_map('strtolower', explode("|", $address));
                $arrayAddress = array_map('trim', $arrayAddress);
                $arrayStreet = [];

                // I check if the last value is the address
                if(strpos($arrayAddress[count($arrayAddress)-1], '+') !== false)
                {
                    $valueAddress = $this->sanitizeLetters(array_pop($arrayAddress));
                    // Array with the data of the street to be blocked
                    $arrayStreet = array_map('trim', explode("+", strtolower($valueAddress)));
                    // Remove array's empty or null elements
                    $arrayStreet = array_filter($arrayStreet);
                }

                // If the street value is not set, I check the other elements of the address
                if(empty($arrayStreet))
                {
                    $check = true;
                }
                else
                {
                    // Check if the street set coincides with the street of order
                    $checkStreetValue = array_intersect($arrayStreet, $arrayOrderStreet);
                    if(count($checkStreetValue) >= count($arrayStreet))
                    {
                        $check = true;
                    }
                    else
                    {
                        /*
                         * If the address is not found in the 'Street' field of the order, I check the 'City' field,
                         * so as to avoid the possibility that the city and address will be reversed.
                         */
                        $checkCityValue = array_intersect($arrayStreet, $arrayOrderCity);
                        if(count($checkCityValue) >= count($arrayStreet))
                        {
                            $check = true;
                        }
                    }
                }

                if($check && !empty($arrayAddress))
                {
                    // Check the other values ​​set for the address to be blocked
                    foreach ($arrayAddress as $val)
                    {
                        $check = $check && in_array(strtolower($this->sanitizeLetters($val)), $orderData);
                    }
                }


                // If all the set values​are present in the shipping address of the order, I block it
                if($check)
                {
                    $addressToBlock = true;
                    break;
                }
            }
        }

        return $addressToBlock;
    }

    private function sanitizeLetters($string)
    {
        $unwanted_array = array('Š'=>'S','š'=>'s','Ž'=>'Z','ž'=>'z','À'=>'A','Á'=>'A','Â'=>'A','Ã'=>'A','Ä'=>'A','Å'=>'A','Æ'=>'A','Ç'=>'C','È'=>'E','É'=>'E','Ê'=>'E','Ë'=>'E','Ì'=>'I','Í'=>'I','Î'=>'I','Ï'=>'I','Ñ'=>'N','Ò'=>'O','Ó'=>'O','Ô'=>'O','Õ'=>'O','Ö'=>'O','Ø'=>'O','Ù'=>'U','Ú'=>'U','Û'=>'U','Ü'=>'U','Ý'=>'Y','Þ'=>'B','ß'=>'B','à'=>'a','á'=>'a','â'=>'a','ã'=>'a','ä'=>'a','å'=>'a','æ'=>'a','ç'=>'c','è'=>'e','é'=>'e','ê'=>'e','ë'=>'e','ì'=>'i','í'=>'i','î'=>'i','ï'=>'i','ð'=>'o','ñ'=>'n','ò'=>'o','ó'=>'o','ô'=>'o','õ'=>'o','ö'=>'o','ø'=>'o','ù'=>'u','ú'=>'u','û'=>'u','ý'=>'y','þ'=>'b','ÿ'=>'y');

        return strtr($string, $unwanted_array);
    }
}